@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -203,7 +200,7 @@ void OutputBuffer()
 	printf( "\t%8lx  %8lx  %8lx  %8lx\n",	pl[0], pl[1], pl[2], pl[3] );
 	pl += 4;
 
-	printf( "\033[33m\t%8lx  lost events.\033[0m\n\n",	pl[0] );
+	printf( "\t%8lx  %8lx  %8lx  %8lx\n",	pl[0], pl[1], pl[2], pl[3] );
 	pl += 4;
 
 	for ( i = 3; i < gUCRequest.logBufferSz / 0x10; ++i )
@@ -219,6 +216,7 @@ void OutputBuffer()
 
 		printf( "%8lx:  %3d  %6ld  %8lx  %8lx\t%s\n",	i * 0x10, lefty, microsec, p1, p2, buffer );
 	}
+	*(UInt32*)gUCRequest.pLogBuffer = 0xFeedBeef;	// get the juices flowing again.
 	return;
 }/* end OutputBuffer */
 
